@@ -58,6 +58,7 @@
 #include "headset.h"
 #include "gateway.h"
 #include "sink.h"
+#include "source.h"
 
 #define AUDIO_INTERFACE "org.bluez.Audio"
 
@@ -362,11 +363,14 @@ static void device_sink_cb(struct audio_device *dev,
 		if (dev->auto_connect) {
 			if (!dev->headset)
 				device_set_state(dev, AUDIO_STATE_CONNECTED);
-			if (priv->hs_state == HEADSET_STATE_DISCONNECTED)
+			else if (priv->hs_state == HEADSET_STATE_DISCONNECTED)
 				device_set_headset_timer(dev);
-			else if (priv->hs_state == HEADSET_STATE_CONNECTED)
+			else if (priv->hs_state == HEADSET_STATE_CONNECTED ||
+					priv->hs_state == HEADSET_STATE_PLAY_IN_PROGRESS ||
+					priv->hs_state == HEADSET_STATE_PLAYING)
 				device_set_state(dev, AUDIO_STATE_CONNECTED);
-		} else if (priv->hs_state != HEADSET_STATE_CONNECTED)
+		} else if (priv->hs_state == HEADSET_STATE_DISCONNECTED ||
+				priv->hs_state == HEADSET_STATE_CONNECTING)
 			device_set_state(dev, AUDIO_STATE_CONNECTED);
 		break;
 	case SINK_STATE_PLAYING:
@@ -417,12 +421,12 @@ static void device_headset_cb(struct audio_device *dev,
 		}
 		if (priv->sink_state == SINK_STATE_DISCONNECTED)
 			device_set_state(dev, AUDIO_STATE_DISCONNECTED);
-		else if (old_state == HEADSET_STATE_CONNECT_IN_PROGRESS &&
+		else if (old_state == HEADSET_STATE_CONNECTING &&
 				(priv->sink_state == SINK_STATE_CONNECTED ||
 				priv->sink_state == SINK_STATE_PLAYING))
 			device_set_state(dev, AUDIO_STATE_CONNECTED);
 		break;
-	case HEADSET_STATE_CONNECT_IN_PROGRESS:
+	case HEADSET_STATE_CONNECTING:
 		device_remove_headset_timer(dev);
 		if (priv->sink_state == SINK_STATE_DISCONNECTED)
 			device_set_state(dev, AUDIO_STATE_CONNECTING);
@@ -437,9 +441,11 @@ static void device_headset_cb(struct audio_device *dev,
 				device_set_state(dev, AUDIO_STATE_CONNECTED);
 			else if (priv->sink_state == SINK_STATE_DISCONNECTED)
 				device_set_avdtp_timer(dev);
-			else if (priv->sink_state == SINK_STATE_CONNECTED)
+			else if (priv->sink_state == SINK_STATE_CONNECTED ||
+					priv->sink_state == SINK_STATE_PLAYING)
 				device_set_state(dev, AUDIO_STATE_CONNECTED);
-		} else if (priv->sink_state != SINK_STATE_CONNECTED)
+		} else if (priv->sink_state == SINK_STATE_DISCONNECTED ||
+				priv->sink_state == SINK_STATE_CONNECTING)
 			device_set_state(dev, AUDIO_STATE_CONNECTED);
 		break;
 	case HEADSET_STATE_PLAY_IN_PROGRESS:
@@ -611,7 +617,7 @@ struct audio_device *audio_device_register(DBusConnection *conn,
 	return dev;
 }
 
-gboolean audio_device_is_connected(struct audio_device *dev,
+gboolean audio_device_is_active(struct audio_device *dev,
 						const char *interface)
 {
 	if (!interface) {
@@ -651,6 +657,9 @@ void audio_device_unregister(struct audio_device *device)
 	if (device->sink)
 		sink_unregister(device);
 
+	if (device->source)
+		source_unregister(device);
+
 	if (device->control)
 		control_unregister(device);
 
@@ -674,6 +683,42 @@ static void auth_cb(DBusError *derr, void *user_data)
 	}
 }
 
+static gboolean auth_idle_cb(gpointer user_data)
+{
+	auth_cb(NULL, user_data);
+	return FALSE;
+}
+
+static gboolean audio_device_is_connected(struct audio_device *dev)
+{
+	if (dev->headset) {
+		headset_state_t state = headset_get_state(dev);
+
+		if (state == HEADSET_STATE_CONNECTED ||
+				state == HEADSET_STATE_PLAY_IN_PROGRESS ||
+				state == HEADSET_STATE_PLAYING)
+			return TRUE;
+	}
+
+	if (dev->sink) {
+		sink_state_t state = sink_get_state(dev);
+
+		if (state == SINK_STATE_CONNECTED ||
+				state == SINK_STATE_PLAYING)
+			return TRUE;
+	}
+
+	if (dev->source) {
+		source_state_t state = source_get_state(dev);
+
+		if (state == SOURCE_STATE_CONNECTED ||
+				state == SOURCE_STATE_PLAYING)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
 int audio_device_request_authorization(struct audio_device *dev,
 					const char *uuid, service_auth_cb cb,
 					void *user_data)
@@ -693,6 +738,11 @@ int audio_device_request_authorization(struct audio_device *dev,
 	if (g_slist_length(priv->auths) > 1)
 		return 0;
 
+	if (audio_device_is_connected(dev)) {
+		g_idle_add(auth_idle_cb, dev);
+		return 0;
+	}
+
 	err = btd_request_authorization(&dev->src, &dev->dst, uuid, auth_cb,
 					dev);
 	if (err < 0) {
