@@ -31,7 +31,8 @@ struct enabled_interfaces {
 	gboolean control;
 };
 
-int audio_manager_init(DBusConnection *conn, GKeyFile *config);
+int audio_manager_init(DBusConnection *conn, GKeyFile *config,
+							gboolean *enable_sco);
 void audio_manager_exit(void);
 
 gboolean server_is_enabled(bdaddr_t *src, uint16_t svc);
@@ -46,4 +47,4 @@ struct audio_device *manager_get_device(const bdaddr_t *src,
 					const bdaddr_t *dst,
 					gboolean create);
 
-gboolean manager_allow_headset_connection(bdaddr_t *src);
+gboolean manager_allow_headset_connection(struct audio_device *device);
