@@ -351,6 +351,9 @@ int hcid_dbus_user_confirm(bdaddr_t *sba, bdaddr_t *dba, uint32_t passkey)
 			(type == 0xff || !(type & 0x01) || remcap == 0x03)) {
 		int dd;
 
+		/* Wait 5 milliseconds before doing auto-accept */
+		usleep(5000);
+
 		dd = hci_open_dev(dev_id);
 		if (dd < 0) {
 			error("Unable to open hci%d", dev_id);
@@ -778,39 +781,6 @@ failed:
 		hci_close_dev(dd);
 }
 
-void hcid_dbus_write_class_complete(bdaddr_t *local)
-{
-	struct btd_adapter *adapter;
-	int dd;
-	uint8_t cls[3];
-	uint16_t dev_id;
-
-	adapter = manager_find_adapter(local);
-	if (!adapter) {
-		error("No matching adapter found");
-		return;
-	}
-
-	dev_id = adapter_get_dev_id(adapter);
-
-	dd = hci_open_dev(dev_id);
-	if (dd < 0) {
-		error("HCI device open failed: hci%d", dev_id);
-		return;
-	}
-
-	if (hci_read_class_of_dev(dd, cls, HCI_REQ_TIMEOUT) < 0) {
-		error("Can't read class of device on hci%d: %s (%d)",
-			dev_id, strerror(errno), errno);
-		hci_close_dev(dd);
-		return;
-	}
-
-	hci_close_dev(dd);
-
-	adapter_set_class(adapter, cls);
-}
-
 void hcid_dbus_write_simple_pairing_mode_complete(bdaddr_t *local)
 {
 	struct btd_adapter *adapter;
